@@ -85,7 +85,8 @@
       res <- with(g1,raster::raster(nrows=rows,ncols=columns))
    }
    raster::extent(res) <- with(g1,c(minx,maxx,miny,maxy))
-   opC <- options(warn=0)
+  # print(c(isPackageInUse=.isPackageInUse()))
+   opC <- options(warn=ifelse(.isPackageInUse(),-1,1))
    raster::crs(res) <- sp::CRS(ursa_crs(g1),doCheckCRSArgs=!FALSE) ## 20220118 doCheck...=TRUE
    options(opC)
    if (isGrid)
